@@ -266,9 +266,21 @@ class VersioningFS(VersionInfoMixIn, HideFS):
     def rename(self, src, dst):
         """Rename a file."""
 
+        # collect a list of paths
+        paths = []
+        if super(VersioningFS, self).isdir(src):
+            for path in super(VersioningFS, self).walkfiles(src):
+                paths.append(path)
+        else:
+            paths = [src]
+
         # rename the file
         super(VersioningFS, self).rename(src, dst)
-        self.__move_snapshot(src, dst)
+
+        # move versions under the path
+        for path in paths:
+            dst_path = path.replace(src, dst)
+            self.__move_snapshot(path, dst_path)
 
     def __move_snapshot(self, src, dst):
         """Move the snapshot associated with a file."""
@@ -296,7 +308,7 @@ class VersioningFS(VersionInfoMixIn, HideFS):
             with temp_snapshot_fs.open('datafile', 'wb') as temp_file:
                 shutil.copyfileobj(source_file, temp_file)
 
-         # snapshot destination directory
+        # snapshot destination directory
         dest_dir = self.snapshot_snap_path(path)
 
         command = ['rdiff-backup', '--parsable-output', '--no-eas',
@@ -321,6 +333,7 @@ class VersioningFS(VersionInfoMixIn, HideFS):
         # close the temp snapshot filesystem
         temp_snapshot_fs.close()
 
+    @synchronize
     def remove_versions_before(self, path, version):
         """Removes snapshots before a specified version.
 
